@@ -114,7 +114,7 @@ class _MemoizeBase:
 
     def bind_key_lifetime(self, raw_key: Tuple[Any, ...], key: Union[int, str]) -> None:
         for raw_key_part in raw_key:
-            if type(raw_key_part).__hash__ is object.__hash__:
+            if (raw_key_part is not None) and (type(raw_key_part).__hash__ is object.__hash__):
                 finalize(raw_key_part, self.reset_key, key)
 
     def default_keygen(self, *args, **kwargs) -> Tuple[Hashable, ...]:
@@ -331,216 +331,260 @@ class _SyncMemoize(_MemoizeBase):
 
 class _Memoize:
     """Decorates a function call and caches return value for given inputs.
+    - If `db_path` is provided, memos will persist on disk and reloaded during initialization.
+    - If `duration` is provided, memos will only be valid for given `duration`.
+    - If `keygen` is provided, memo hash keys will be created with given `keygen`.
+    - If `size` is provided, LRU memo will be evicted if current count exceeds given `size`.
 
-    If 'db' is provided, memoized values will be saved to disk and reloaded during initialization.
+    ### Examples
 
-    If 'duration' is provided, memoize will only retain return values for up to given 'duration'.
+    - Body will run once for unique input `bar` and result is cached.
+        ```python3
+        @memoize
+        def foo(bar) -> Any: ...
 
-    If 'keygen' is provided, memoize will use the function to calculate the memoize hash key.
+        foo(1)  # Function actually called. Result cached.
+        foo(1)  # Function not called. Cached result returned.
+        foo(2)  # Function actually called. Result cached.
+        ```
 
-    If 'size' is provided, memoize will only retain up to 'size' return values.
+    - Same as above, but async.
+        ```python3
+        @memoize
+        async def foo(bar) -> Any: ...
 
-    A warning about arguments inheriting `object.__hash__`:
+        # Concurrent calls from the same event loop are safe. Only one call is generated. The
+        # other nine calls in this example wait for the result.
+        await asyncio.gather(*[foo(1) for _ in range(10)])
+        ```
 
-        It doesn't make sense to keep a memo if it's impossible to generate the same input again.
-        Inputs that inherit the default `object.__hash__` are unique based on their id, and thus,
-        their location in memory. If such inputs are garbage-collected, they are assumed to be gone
-        forever. For that reason, when those inputs are garbage collected, `memoize` will drop memos
-        created using those inputs.
+    - Classes may be memoized.
+        ```python3
+        @memoize
+        Class Foo:
+            def init(self, _): ...
 
-        Here are some common patterns where this behaviour will not cause any problems.
+        Foo(1)  # Instance is actually created.
+        Foo(1)  # Instance not created. Cached instance returned.
+        Foo(2)  # Instance is actually created.
+        ```
 
-            - Basic immutable types that have specific, consistent hash functions (int, str, etc.).
-                @memoize
-                def foo(a: int, b: str, c: Tuple[int, ...], d: range) -> Any: ...
+    - Calls `foo(1)`, `foo(bar=1)`, and `foo(1, baz='baz')` are equivalent and only cached once.
+        ```python3
+        @memoize
+        def foo(bar, baz='baz'): ...
+        ```
 
-                foo(1, 'bar', (1, 2, 3), range(42))  # Function called. Result cached.
-                foo(1, 'bar', (1, 2, 3), range(42))  # Function not called. Cached result returned.
+    - Only 2 items are cached. Acts as an LRU.
+        ```python3
+        @memoize(size=2)
+        def foo(bar) -> Any: ...
 
-            - Classmethods rely on classes, which inherit from `object.__hash__`. However, classes
-                are almost never garbage collected until a process exits so memoize will work as
-                expected.
+        foo(1)  # LRU cache order [foo(1)]
+        foo(2)  # LRU cache order [foo(1), foo(2)]
+        foo(1)  # LRU cache order [foo(2), foo(1)]
+        foo(3)  # LRU cache order [foo(1), foo(3)], foo(2) is evicted to keep cache size at 2
+        ```
 
-                class Foo:
+    - Items are evicted after 1 minute.
+        ```python3
+        @memoize(duration=datetime.timedelta(minutes=1))
+        def foo(bar) -> Any: ...
 
-                    @classmethod
-                    @memoize
-                    def bar(cls) -> Any: ...
+        foo(1)  # Function actually called. Result cached.
+        foo(1)  # Function not called. Cached result returned.
+        sleep(61)
+        foo(1)  # Function actually called. Cached result was too old.
+        ```
 
-                foo = Foo()
-                foo.bar()  # Function called. Result cached.
-                foo.bar()  # Function not called. Cached result returned.
+    - Memoize can be explicitly reset through the function's `.memoize` attribute
+        ```python3
+        @memoize
+        def foo(bar) -> Any: ...
 
-                del foo  # Memo not cleared since lifetime is bound to class Foo.
+        foo(1)  # Function actually called. Result cached.
+        foo(1)  # Function not called. Cached result returned.
+        foo.memoize.reset()
+        foo(1)  # Function actually called. Cache was emptied.
+        ```
 
-                foo = Foo()
-                foo.bar()  # Function not called. Cached result returned.
-                foo.bar()  # Function not called. Cached result returned.
+    - Current cache length can be accessed through the function's `.memoize` attribute
+        ```python3
+        @memoize
+        def foo(bar) -> Any: ...
 
-            - Long-lasting object instances that inherit from `object.__hash__`.
+        foo(1)
+        foo(2)
+        len(foo.memoize)  # returns 2
+        ```
 
-                class Foo:
+    - Alternate memo hash function can be specified. The inputs must match the function's.
+        ```python3
+        Class Foo:
+            @memoize(keygen=lambda self, a, b, c: (a, b, c))  # Omit 'self' from hash key.
+            def bar(self, a, b, c) -> Any: ...
 
-                    @memoize
-                    def bar(self) -> Any: ...
+        a, b = Foo(), Foo()
 
-                foo = Foo()
-                foo.bar()  # Function called. Result cached.
-                foo.bar()  # Function not called. Cached result returned.
+        # Hash key will be (a, b, c)
+        a.bar(1, 2, 3)  # LRU cache order [Foo.bar(a, 1, 2, 3)]
 
-                del foo  # Memo is cleared since lifetime is bound to instance foo.
+        # Hash key will again be (a, b, c)
+        # Be aware, in this example the returned result comes from a.bar(...), not b.bar(...).
+        b.bar(1, 2, 3)  # Function not called. Cached result returned.
+        ```
 
-                foo = Foo()
-                foo.bar()  # Function called. Result cached.
-                foo.bar()  # Function not called. Cached result returned.
+    - If part of the returned key from keygen is awaitable, it will be awaited.
+        ```python3
+        async def awaitable_key_part() -> Hashable: ...
 
-        Here are common patterns that will not behave as desired (for good reason).
+        @memoize(keygen=lambda bar: (bar, awaitable_key_part()))
+        async def foo(bar) -> Any: ...
+        ```
 
-            - Using ephemeral objects that inherit from `object.__hash__`. Firstly, these inputs
-                will only hash equally sometimes, by accident, if their id is recycled from a
-                previously deleted input. Secondly, we delete memos based on inputs that inherit
-                from `object.__hash__` at the same time as that input is garbage collected, so
-                generating the memo is wasted effort.
+    - If the memoized function is async and any part of the key is awaitable, it is awaited.
+        ```python3
+        async def morph_a(a: int) -> int: ...
 
-                # Inherits object.__hash__
-                class Foo: ...
+        @memoize(keygen=lambda a, b, c: (morph_a(a), b, c))
+        def foo(a, b, c) -> Any: ...
+        ```
 
-                @memoize
-                def bar(foo: Foo) -> Any: ...
-
-                bar(Foo())  # Memo is immediately deleted since Foo() is garbage collected.
-                bar(Foo())  # Same as previous line. Memo is immediately deleted.
-
-    Examples:
-
-        - Body will run once for unique input 'bar' and result is cached.
+    - Properties can be memoized.
+        ```python3
+        Class Foo:
+            @property
             @memoize
-            def foo(bar) -> Any: ...
+            def bar(self) -> Any: ...
 
-            foo(1)  # Function actually called. Result cached.
-            foo(1)  # Function not called. Cached result returned.
-            foo(2)  # Function actually called. Result cached.
+        a = Foo()
+        a.bar  # Function actually called. Result cached.
+        a.bar  # Function not called. Cached result returned.
+
+        b = Foo() # Memoize uses 'self' parameter in hash. 'b' does not share returns with 'a'
+        b.bar  # Function actually called. Result cached.
+        b.bar  # Function not called. Cached result returned.
+        ```
+
+    - Be careful with eviction on instance methods. Memoize is not instance-specific.
+        ```python3
+        Class Foo:
+            @memoize(size=1)
+            def bar(self, baz) -> Any: ...
+
+        a, b = Foo(), Foo()
+        a.bar(1)  # LRU cache order [Foo.bar(a, 1)]
+        b.bar(1)  # LRU cache order [Foo.bar(b, 1)], Foo.bar(a, 1) is evicted
+        a.bar(1)  # Foo.bar(a, 1) is actually called and cached again.
+        ```
+
+    - Values can persist to disk and be reloaded when memoize is initialized again.
+        ```python3
+        @memoize(db_path=Path.home() / '.memoize')
+        def foo(a) -> Any: ...
+
+        foo(1)  # Function actually called. Result cached.
+
+        # Process is restarted. Upon restart, the state of the memoize decorator is reloaded.
+
+        foo(1)  # Function not called. Cached result returned.
+        ```
+
+    - If not applied to a function, calling the decorator returns a partial application.
+        ```python3
+        memoize_db = memoize(db_path=Path.home() / '.memoize')
+
+        @memoize_db(size=1)
+        def foo(a) -> Any: ...
+
+        @memoize_db(duration=datetime.timedelta(hours=1))
+        def bar(b) -> Any: ...
+        ```
+
+    - Comparison equality does not affect memoize. Only hash equality matters.
+        ```python3
+        # Inherits object.__hash__
+        class Foo:
+            # Don't be fooled. memoize only cares about the hash.
+            def __eq__(self, other: Foo) -> bool:
+                return True
+
+        @memoize
+        def bar(foo: Foo) -> Any: ...
+
+        foo0, foo1 = Foo(), Foo()
+        assert foo0 == foo1
+        bar(foo0)  # Function called. Result cached.
+        bar(foo1)  # Function called again, despite equality, due to different hash.
+        ```
+
+    ### A warning about arguments that inherit `object.__hash__`:
+
+    It doesn't make sense to keep a memo if it's impossible to generate the same input again. Inputs
+    that inherit the default `object.__hash__` are unique based on their id, and thus, their
+    location in memory. If such inputs are garbage-collected, they are gone forever. For that
+    reason, when those inputs are garbage collected, `memoize` will drop memos created using those
+    inputs.
+
+    - Memo lifetime is bound to the lifetime of any arguments that inherit `object.__hash__`.
+        ```python3
+        # Inherits object.__hash__
+        class Foo:
+            ...
+
+        @memoize
+        def bar(foo: Foo) -> Any: ...
+
+        bar(Foo())  # Memo is immediately deleted since Foo() is garbage collected.
+
+        foo = Foo()
+        bar(foo)  # Memo isn't deleted until foo is deleted.
+        del foo  # Memo is deleted at the same time as foo.
+        ```
+
+    - Types that have specific, consistent hash functions (int, str, etc.) won't cause problems.
+        ```python3
+        @memoize
+        def foo(a: int, b: str, c: Tuple[int, ...], d: range) -> Any: ...
+
+        foo(1, 'bar', (1, 2, 3), range(42))  # Function called. Result cached.
+        foo(1, 'bar', (1, 2, 3), range(42))  # Function not called. Cached result returned.
+        ```
+
+    - Classmethods rely on classes, which inherit from `object.__hash__`. However, classes are
+      almost never garbage collected until a process exits so memoize will work as expected.
+
+        ```python3
+        class Foo:
+          @classmethod
+          @memoize
+          def bar(cls) -> Any: ...
+
+        foo = Foo()
+        foo.bar()  # Function called. Result cached.
+        foo.bar()  # Function not called. Cached result returned.
+
+        del foo  # Memo not cleared since lifetime is bound to class Foo.
+
+        foo = Foo()
+        foo.bar()  # Function not called. Cached result returned.
+        foo.bar()  # Function not called. Cached result returned.
+        ```
+
+    - Long-lasting object instances that inherit from `object.__hash__`.
+
+        ```python3
+        class Foo:
 
-        - Same as above, but async.
             @memoize
-            async def foo(bar) -> Any: ...
+            def bar(self) -> Any: ...
 
-            # Concurrent calls from the same event loop are safe. Only one call is generated. The
-            other nine calls in this example wait for the result.
-            await asyncio.gather(*[foo(1) for _ in range(10)])
+        foo = Foo()
+        foo.bar()  # Function called. Result cached.
 
-        - Classes may be memoized.
-            @memoize
-            Class Foo:
-                def init(self, _): ...
-
-            Foo(1)  # Instance is actually created.
-            Foo(1)  # Instance not created. Cached instance returned.
-            Foo(2)  # Instance is actually created.
-
-        - Calls to foo(1), foo(bar=1), and foo(1, baz='baz') are equivalent and only cached once
-            @memoize
-            def foo(bar, baz='baz'): ...
-
-        - Only 2 items are cached. Acts as an LRU.
-            @memoize(size=2)
-            def foo(bar) -> Any: ...
-
-            foo(1)  # LRU cache order [foo(1)]
-            foo(2)  # LRU cache order [foo(1), foo(2)]
-            foo(1)  # LRU cache order [foo(2), foo(1)]
-            foo(3)  # LRU cache order [foo(1), foo(3)], foo(2) is evicted to keep cache size at 2
-
-       - Items are evicted after 1 minute.
-            @memoize(duration=datetime.timedelta(minutes=1))
-            def foo(bar) -> Any: ...
-
-            foo(1)  # Function actually called. Result cached.
-            foo(1)  # Function not called. Cached result returned.
-            sleep(61)
-            foo(1)  # Function actually called. Cached result was too old.
-
-        - Memoize can be explicitly reset through the function's 'memoize' attribute
-            @memoize
-            def foo(bar) -> Any: ...
-
-            foo(1)  # Function actually called. Result cached.
-            foo(1)  # Function not called. Cached result returned.
-            foo.memoize.reset()
-            foo(1)  # Function actually called. Cache was emptied.
-
-        - Current cache size can be accessed through the function's 'memoize' attribute
-            @memoize
-            def foo(bar) -> Any: ...
-
-            foo(1)
-            foo(2)
-            len(foo.memoize)  # returns 2
-
-        - Memoization hash keys can be generated from a non-default function:
-            @memoize(keygen=lambda a, b, c: (a, b, c))
-            def foo(a, b, c) -> Any: ...
-
-        - If part of the returned key from keygen is awaitable, it will be awaited.
-            async def await_something() -> Hashable: ...
-
-            @memoize(keygen=lambda bar: (bar, await_something()))
-            async def foo(bar) -> Any: ...
-
-        - Properties can be memoized
-            Class Foo:
-                @property
-                @memoize
-                def bar(self): -> Any: ...
-
-            a = Foo()
-            a.bar  # Function actually called. Result cached.
-            a.bar  # Function not called. Cached result returned.
-
-            b = Foo() # Memoize uses 'self' parameter in hash. 'b' does not share returns with 'a'
-            b.bar  # Function actually called. Result cached.
-            b.bar  # Function not called. Cached result returned.
-
-        - Be careful with eviction on methods.
-            Class Foo:
-                @memoize(size=1)
-                def bar(self, baz): -> Any: ...
-
-            a, b = Foo(), Foo()
-            a.bar(1)  # LRU cache order [Foo.bar(a, 1)]
-            b.bar(1)  # LRU cache order [Foo.bar(b, 1)], Foo.bar(a, 1) is evicted
-            a.bar(1)  # Foo.bar(a, 1) is actually called and cached again.
-
-        - The default memoize key generator can be overridden. The inputs must match the function's.
-            Class Foo:
-                @memoize(keygen=lambda self, a, b, c: (a, b, c))
-                def bar(self, a, b, c) -> Any: ...
-
-            a, b = Foo(), Foo()
-
-            # Hash key will be (a, b, c)
-            a.bar(1, 2, 3)  # LRU cache order [Foo.bar(a, 1, 2, 3)]
-
-            # Hash key will again be (a, b, c)
-            # Be aware, in this example the returned result comes from a.bar(...), not b.bar(...).
-            b.bar(1, 2, 3)  # Function not called. Cached result returned.
-
-        - If the memoized function is async and any part of the key is awaitable, it is awaited.
-            async def morph_a(a: int) -> int: ...
-
-            @memoize(keygen=lambda a, b, c: (morph_a(a), b, c))
-            def foo(a, b, c) -> Any: ...
-
-        - Values can persist to disk and be reloaded when memoize is initialized again.
-
-            @memoize(db_path=Path.home() / '.memoize')
-            def foo(a) -> Any: ...
-
-            foo(1)  # Function actually called. Result cached.
-
-            # Process is restarted. Upon restart, the state of the memoize decorator is reloaded.
-
-            foo(1)  # Function not called. Cached result returned.
+        # foo instance is kept around somewhere and used later.
+        foo.bar()  # Function not called. Cached result returned.
+        ```
     """
 
     _all_decorators = WeakSet()
@@ -553,7 +597,7 @@ class _Memoize:
             duration: Optional[Union[int, float, timedelta]] = None,
             keygen: Optional[Keygen] = None,
             size: Optional[int] = None,
-    ):
+    ) -> Union[Decoratee]:
         if _decoratee is None:
             return partial(memoize, db_path=db_path, duration=duration, keygen=keygen, size=size)
 
